import java.awt.Cursor;
import java.awt.Graphics2D;
import java.util.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.*;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.Timer;

/**
 * @author  Oscar Bahamonde & Luis Vera
 */
public class myNetwork implements ActionListener{
	public static final double DEFAUL_DELTA = 0.0001;
	public static final double DEAFAUL_REFRESH = 0.003;
	public static final double DEFAULT_CONGWIN = 128;
    private Vector<Source> sources;		//Vector de nodos.
    private CongestionControlPanel2 ccp2;	//Panel de control de congestion.
    private Timer pase;
    private double refreshPeriod;
    private double delta_t;
    private double t;
    private double totalCongWin;	//Ventana de congestion de la red.
    private double RTT = 1;
    private boolean deletenable = false;
    private GraphicPanel gp;	//Panel de grafico.
    public static int WIDTH = 1024;
	public static int HEIGHT = 600;
	public static int X_ORIGEN = (int)(WIDTH*0.1);
	public static int Y_ORIGEN = (int)(HEIGHT*0.9);
	public static AffineTransform SPACE_TRANSFORM;
	public static AffineTransform SPACE_INVERSE_TRANSFORM;
	public static Line2D.Double SCALE_LINE;
	private static double AXES_SCALE = 100.0;

	static {
		SPACE_TRANSFORM = AffineTransform.getTranslateInstance(X_ORIGEN, Y_ORIGEN);
		SPACE_TRANSFORM.scale(AXES_SCALE,-AXES_SCALE);
		try {
			SPACE_INVERSE_TRANSFORM = SPACE_TRANSFORM.createInverse();
		}catch (NoninvertibleTransformException e){ 
			System.exit(0);
		}
		SCALE_LINE = new Line2D.Double(300,15, 300+AXES_SCALE,15);
	}
    
	/**
	 * Constructor.	Inicializa una nueva red (Netowrk).
	 */
    public myNetwork () {
    	t = 0;	//Empiza con tiempo igual a 0.
    	refreshPeriod = 0.05;	//Tiempo de refresco de la pantalla.
    	delta_t = 0.0001;
    	pase = new Timer((int)(refreshPeriod*1000), this);
        sources = new Vector<Source>();
        ccp2 = null;
        totalCongWin = 128;
    }
    
    /**
     * Setea el panel de control de congestion.
     * @param ccp2	
     */
    public void setPanel (CongestionControlPanel2 ccp2 ) {
        this.ccp2 = ccp2;
    }
    
    /**
     * Setea el panel que contiene el grafico.
     * @param gp	Panel grafico.
     */
    public void setGraphicPanel (GraphicPanel gp) {
    	this.gp = gp;
    }
    
    /**
     * Agregar un nodo a la red.
     * @param s	Nodo.
     */
    public void addSource (final Source s) {
        sources.add(s);		//Agregar nodo al vector de nodos.
        if ((ccp2 != null)) {	//Si existe el panel.
        	JButton acks = new JButton("3 Ack's");	//Crear boton de 3 Acks repetidos.
        	acks.addActionListener( new ActionListener (){	//Listener del boton.
        	       public void actionPerformed(ActionEvent event) {
        	         s.threeAck();	//Nodo asociado recibe 3 Acks repetidos.
        	       };
        	      });
        	JButton timeout = new JButton("Timeout");	//Crear boton de Timeout.
        	timeout.addActionListener( new ActionListener (){	//Listener del boton.
     	       public void actionPerformed(ActionEvent event) {
     	         s.timeOut();	//Nodo recibe Timeout.
     	       };
     	      });
        	if (s.getID() == 1) {
        		acks.setBounds(130, s.getID()*20 + 40, 100, 20);	//Posicionar boton.
        		timeout.setBounds(130, s.getID()*20 + 65, 100, 20);	//Posicionar boton.
        	}
        	else {
        		acks.setBounds(130, s.getID()*60, 100, 20);
        		timeout.setBounds(130, s.getID()*60 + 25, 100, 20);
        	}
        	ccp2.add(acks);	//Agregar boton al panel.
        	ccp2.add(timeout);	//Agregar boton al panel.
            ccp2.repaint();	//Redibujar panel.
        }
    }
    
    /**
     * Elimina un nodo.
     * @param s	Nodo.
     */
    public void removeSource(Source s) {
		sources.remove(s);	//Elimina un nodo del vector de nodos.
	}
    
    /**
	 * @param  tcw
	 * @uml.property  name="totalCongWin"
	 */
    public void setTotalCongWin (double tcw) {
    	totalCongWin = tcw;
    }
    
    /**
     * Establecer Round trip time.
	 * @param  rtt	Round trip time.
	 */
    public void setRTT (double rtt) {
        RTT = rtt;
    }
    
    /**
	 * @return	Round trip time.
	 */
    public double getRTT () {
        return RTT;
    }
    
    /**
     * 
     * @return	Suma de la ventana de cada nodo.
     */
    public double getNextSumCongWin() {
        double sumNextCongWin = 0;
        for (Source s: sources)
            sumNextCongWin += s.nextIncrement();
        return sumNextCongWin;
    }
    
    /**
     * Establecer delta de tempo.
	 * @param  delta	Delta de tiempo.
	 */
    public void setDelta_t(double delta) {
    	delta_t = delta;
    }
    
    /**
     * Dibujar cada nodo.
     * @param g2d	Grafico que dibuja el nodo.
     */
    public void paintComponent (Graphics2D g2d) {
        for (Source s: sources)
            ((Drawable)s).draw(g2d);
    }
    
    /**
     * Iniciar la simulacion.
     */
    public void start() {
        if (pase.isRunning())
            return;
        pase.start();
    }
    
    /**
     * Detener la simulacion.
     */
    public void stop() {
    	pase.stop();
    }
    
    /**
     * Esteblecer el tiempo de refresco.
	 * @param  rp	Tiempo de refresco.
	 */
    public void setRefreshPeriod (double rp) {
		refreshPeriod = rp;
		pase.setDelay((int)(refreshPeriod*1000)); 
	}

    /**
     * @param event
     */
    public void actionPerformed(ActionEvent event) {
        double nextStep = t + refreshPeriod;
        while (t < nextStep) {
        	if ((t%RTT) == 0) {
        		if (getNextSumCongWin() > totalCongWin) {
        			for (Source s: sources){
        				s.threeAck();
        				if (s.getRecentlyAdded())
        					s.setRecentlyAdded(false);
        			}
        		}
        		else {
        			for (Source s: sources) {
        				s.increment();
        			}
        		}
        	}
        	t = (double)Math.round((t + delta_t)*10000)/10000;
        }
        if (gp != null)
        	gp.addSeries(sources, t, totalCongWin);
        ccp2.repaint();
    }
    
    /**
     * Detectar posicion de un nodo en la pantalla.
     * @param d	Posicion x del nodo.
     * @param f	Posicion y del nodo.
     * @return	GSource.
     */
    public Drawable find(double d, double f) {
		for (Source s: sources)
			if (s instanceof Drawable){
				Drawable ds = (Drawable) s;
				if (ds.contains(d,f)) return ds;
			}
		return null;
	}
    
    /**
     * Activar modo para eliminar nodo.
     */
	public void enableDelete() {
		deletenable = true;
		ccp2.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	}
	
	/**
	 * Desactivar modo para eliminar nodo.
	 */
	public void disableDelete() {
		deletenable = false;
		ccp2.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	/**
	 * Detecta se esta activado el modo eliminar.
	 * @return	deletenable (true or false).
	 */
	public boolean isDeletenable() {
		return deletenable;
	}
}
